;;;; Editing improvements

;; fix Emacs' definition of a sentence
(setq sentence-end-double-space nil)

;; the utter basics
(global-set-key (kbd "C-h") 'backward-delete-char)
(define-key isearch-mode-map (kbd "C-h") 'isearch-delete-char)
(global-set-key (kbd "C-w") 'backward-kill-word)
(defun cp-kill-line-0 ()
  (interactive)
  (kill-line 0))
(global-set-key (kbd "C-u") 'cp-kill-line-0)

(global-unset-key (kbd "C-x C-r"))
(cp-set-keys
 :bindings
 ;; whole-line-or-region
 `((,(kbd "C-y") whole-line-or-region-yank)
   (,(kbd "M-w") whole-line-or-region-kill-ring-save)
   (,(kbd "C-x C-r C-i") string-insert-rectangle)
   (,(kbd "C-x C-r C-r") replace-rectangle)
   (,(kbd "C-x C-r C-k") kill-rectangle)

   ;; A better solution would be to hack kill-line/kill-visual-line to
   ;; kill the region if a region is active. The flexibility of those
   ;; functions is very desirable - you can delete from point to line
   ;; end, clear a line to enter some other text on it (C-a C-k/"a k"),
   ;; or delete one or more whole lines (C-1 C-k/"1 k" or C-<num>
   ;; C-k/"<num> k").
   (,(kbd "C-S-k") whole-line-or-region-kill-region)

   (,(kbd "C-,") backward-paragraph)
   (,(kbd "C-.") forward-paragraph)
   (,(kbd "C-<") beginning-of-buffer)
   (,(kbd "C->") end-of-buffer)
   (,(kbd "C-;") delete-horizontal-space)))

(defun cp/downcase-dwim (arg)
  "Like `downcase-word', but if region is active, run
`downcase-region' instead. Unlike `downcase-region', rectangular
regions are handled correctly as well.")

;; open-line should always move to the beginning of the current line
;; first, so one can run it anywhere. I also want it to indent it to
;; the next or the previous line...

;; These affect org-meta-return and org-insert-heading-respect-content
;;

;; (defadvice open-line
;;     (before open-line-bol activate)
;;   (beginning-of-visual-line))
;; (defadvice open-line
;;     (after open-line-indent activate)
;;   (if (not (looking-at "\\** "))
;;       (indent-for-tab-command)))

;; (global-set-key (kbd "C-o") 'open-line)
(defun cp/open-line ()
  "The opposite of `cp/open-line-before'. Start a new line below
the current line, regardless of where point is in the current
line. Will not affect the content of the current line. Applies
the correct indentation according to the mode."
  (interactive)
  (progn
    (end-of-line)
    (electric-indent-just-newline t)
    (indent-according-to-mode)))
(defun cp/open-line-before (arg)
  "Like `open-line' but a bit more sane.

In org-mode, run org-open-line.

In other modes, insert a newline above the current line,
regardless of where point is in current line. Will not affect
content of current line. Applies the correct indentation
according to the mode."
  (interactive "p")
  (if (and (equal major-mode 'org-mode)
           (org-at-table-p))
      (org-open-line arg)
    (progn

      (beginning-of-line)
      (electric-indent-just-newline t)
      (forward-char -1)
      (indent-according-to-mode))))
(global-set-key (kbd "C-o") 'cp/open-line-before)

;; I wish you could press e.g. S-o in god-mode to get C-S-o.
;; 10/10 would make life better.
(global-set-key (kbd "C-S-o") 'cp/open-line)

(setq dired-bind-jump nil)
(global-set-key (kbd "C-x C-j") 'join-line)

;; C-z (suspend-frame) is utterly useless and disruptive - good place to put universal-argument
(global-unset-key (kbd "C-z"))
(global-set-key (kbd "C-z") 'universal-argument)
(global-set-key (kbd "C-x C-z") 'repeat)
(global-set-key (kbd "C-x C-;") 'repeat-complex-command)

;; (defun cp-kill-line (&optional arg)
;;   (interactive)
;;   (if arg
;;       (if mark-active
;;           (kill-region arg)
;;         (kill-line arg))
;;     (if mark-active
;;       (kill-region)
;;       (kill-line))))
;; (global-set-key (kbd "C-k") 'cp-kill-line)
;; Chris Done (god-mode) recommends this, but Emacs' repeat command is almost useless...
;; (global-set-key (kbd "C-.") 'repeat)

;;;; I like the transpose commands
(global-unset-key (kbd "C-t"))
(cp-set-keys
 :bindings
 `((,(kbd "C-t C-c") transpose-chars)
   (,(kbd "C-t C-w") transpose-words)
   (,(kbd "C-t C-l") transpose-lines)
   (,(kbd "C-t C-s") transpose-sentences)
   (,(kbd "C-t C-e") transpose-sexps)
   (,(kbd "C-t C-p") transpose-paragraphs)))

(global-set-key (kbd "C-M-h") 'er/expand-region)

(use-package dot-mode
  :hook (text-mode prog-mode LilyPond-mode)
  :bind (:map dot-mode-map
              ("C-."   . nil)
              ("M-."   . dot-mode-execute)
              ("C-M-." . nil)))

(add-hook 'before-save-hook 'delete-trailing-whitespace)

(require 'flex-isearch)
(flex-isearch-mode 1)

(cp-set-keys
 :bindings
 `((,(kbd "C-M-s") flex-isearch-forward)
   (,(kbd "C-M-r") flex-isearch-backward)))

(with-eval-after-load 'iedit
  (define-key iedit-mode-keymap (kbd "C-h") 'backward-delete-char))

(with-eval-after-load 'markdown-mode
  (defun cp/copy-md-link (prefix-arg)
    "Copy address of Markdown link after point in the current line.

If there is no link in the current line, or if the region is
active, or with a prefix arg - call
whole-line-or-region-kill-ring-save instead."
    (interactive "P")
    (save-excursion
      (if (or (use-region-p)
              prefix-arg
              (not (re-search-line
                    ;; 2018-03-21T22:47:55+0530 - fix bug where a line with parenthesized text would not be copied
                    ;; "\("
                    ;; 2018-07-22T10:20:03+0530
                    ;; "\\[.*?\\](.*?)"
                    "\\[.*?\\]("
                    )))
          (whole-line-or-region-kill-ring-save prefix-arg)
        (let ((point-a (point)))
          (forward-char -1)
          (forward-sexp)
          (copy-region-as-kill point-a (- (point) 1))))))
  (cp-set-keys
   :keymap markdown-mode-map
   :bindings
   `((,(kbd "M-w") cp/copy-md-link)
     (,(kbd "TAB") markdown-cycle)
     (,(kbd "C-c C-.") markdown-demote)
     (,(kbd "C-c C-,") markdown-promote)
     (,(kbd "C-c C-l") markdown-insert-link))))

(defun cp/copy-bus-entry ()
  "For personal use, when working between Markdown and the OSM wiki."
  (interactive)
  (let ((point-a (region-beginning))
        (point-b (region-end))
        (point-b-line (line-number-at-pos)))
    (query-replace "[ ]" "☐" nil point-a point-b)
    (query-replace "[x]" "☑" nil point-a point-b)
    (query-replace-regexp "^- " "" nil point-a point-b)
    (goto-char (point-min))
    (forward-line (- point-b-line 1))
    (copy-region-as-kill point-a (point-at-bol))))

;; 2018-08-21T03:41:47+0530
(defun cp/copy-md-link (prefix-arg)
  "Copy address of Markdown link after point in the current line.

If there is no link in the current line, or if the region is
active, or with a prefix arg - call
whole-line-or-region-kill-ring-save instead."
  (interactive "P")
  (save-excursion
    (cond
     ((or (use-region-p) prefix-arg)
      (whole-line-or-region-kill-ring-save prefix-arg))
     ((re-search-line "\\[.*?\\](")
      (let ((point-a (point)))
        (forward-char -1)
        (forward-sexp)
        (copy-region-as-kill point-a (- (point) 1))))
     ((re-search-line "http")
      (kill-new (thing-at-point 'url)))
     (t (whole-line-or-region-kill-ring-save prefix-arg)))))

;; 2018-08-28T22:50:43+0530
(defun cp/copy-buffer ()
  (interactive)
  (copy-region-as-kill (point-min) (point-max)))

(use-package multiple-cursors
  :bind (("C-M-," . mc/mark-previous-like-this)
         ("C-M-." . mc/mark-next-like-this)
         ("C-M-/" . mc/mark-all-like-this)))
