;; -*- lexical-bindiing: t; -*-

;;;; File management

(add-to-list 'load-path "~/.emacs.d/elisp-git/dired-toggle-sudo/")
(add-to-list 'load-path "~/.emacs.d/elisp-git/project-explorer/")

(use-package dired
  :init (add-hook 'dired-mode-hook 'turn-on-launch-mode)
  :config
  (setq dired-listing-switches
        ;; by date
        ;; "-cgGhlt --group-directories-first --time-style=long-iso"
        ;; by name
        "-cgGhl --group-directories-first --time-style=long-iso"
        ;; no -h
        ;; "-cgGl --group-directories-first --time-style=long-iso"

        ;; by date, no --group-directories-first
        ;; "-cgGhlt --time-style=long-iso"
        )
  :bind (("<f2> s"  . cp/sxiv)
         :map dired-mode-map
         ("W"       . wdired-change-to-wdired-mode)
         ;; after learning that this copies whole paths with null
         ;; argument, this became a whole lot more useful
         ("C-w"     . dired-copy-filename-as-kill)
         ("C-c C-f" . cp/corresponding-text-file)
         ("h"       . cp/dired-toggle-hidden)
         ([mouse-2] . cp/dired-launch-or-open)
         ("C-j"     . launch-files-dired)
         ("j"       . launch-files-dired)
         ("M-s r"   . dired-do-query-replace-regexp)
         ("M-s s"   . dired-do-isearch-regexp)))

(use-package dired-x
  :commands dired-jump
  :bind ("C-x C-d" . dired-jump))

;; TODO - make launch-file suggest the path at point by default
(use-package launch
  :commands turn-on-launch-mode launch-files-dired
  :bind ("s-l" . launch-file))

(defun cp/dired-marked-files-p ()
  "Return t if there are marked files in the current Dired
buffer. With no marked files, or if not in a Dired buffer, return
nil."
  ;; BUG - doesn't work as intended with non '*' markers (e.g. C or D)
  (interactive)
  (if (equal major-mode 'dired-mode)
      (if (save-excursion
            (goto-char (point-min))
            (re-search-forward dired-re-mark nil t))
          t
        nil)
    nil))

(defun cp/sxiv ()
  "Run sxiv."
  (interactive)

  ;; TODO -

  ;; 1. let user edit options when called with prefix arg
  ;;    - use magit-popup for options! ❤
  ;; 2. when called with -o, parse the file names given by sxiv and
  ;;    mark them in dired buffers (creating the dired buffers if
  ;;    necessary)
  ;; 3. when called without arguments and without markers, run only
  ;;    for _files_ in the current directory. It's easy to mark
  ;;    files/hit t (select all)/give a prefix or null argument before
  ;;    calling it.

  ;; BUG - doesn't work as intended with non '*' markers (e.g. C or D)
  (let ((paths (if (cp/dired-marked-files-p)
                   (->> (dired-get-marked-files)
                        (--map (concat "\"" it "\""))
                        (--reduce-from (concat acc " " it) ""))
                 ;; not sure if I should use * or just .
                 "*")))
    (async-shell-command (concat "sxiv -afor -- " paths))))

(defun cp/open-random-file (&optional find-args dir cmd)
  (interactive)
  (let* ((find-args (if find-args find-args " -type f "))
         (file-name
          (->> (concat "find "
                       (if dir
                           dir
                         (read-directory-name "Directory: "
                                              (if file-name-history
                                                  (car file-name-history)
                                                default-directory)
                                              nil
                                              t))
                       " "
                       find-args
                       " | shuf | sed 1q")
               (shell-command-to-string )
               (replace-regexp-in-string "\n" ""))))
    (if cmd
        (async-shell-command (concat cmd " \"" file-name "\""))
      (find-file file-name))))

(with-eval-after-load 'project-explorer
  (global-set-key (kbd "<f5> e") 'project-explorer-toggle))

;; (require 'sudo-edit)

(defun cp/dired-ffmpeg (&optional format)
  (interactive)
  (let ((files    (-map #'shell-quote-argument (dired-get-marked-files t)))
        (format   (if format format "mp3")))
    (--map (let* ((fbasename (file-name-base it))
                  (new-name (concat fbasename "." format)))
             (call-process "ffmpeg" nil "*dired-ffmpeg*" t
                           "-i" it "-ac" "2" "-f" format new-name))
           files)))

(defvar cp/ffmpeg-max-processes 1
  "Maximum number of simultaneous ffmpeg processes run by
`cp/dired-do-ffmpeg'. See also
`cp/ffmpeg-process-recount-interval'.")

(defvar cp/ffmpeg-process-recount-interval 10
  "Time interval to check number of processes after, and maybe
start new processes. See TIME argument of `run-at-time' for
possible values.")

(defun cp/ffmpeg-process-count ()
  (->>
   (-map 'process-name (process-list))
   (--count (string-match-p "^dired-ffmpeg" it))))

(defun cp/maybe-call-ffmpeg (shell-args)
  (if (< (cp/ffmpeg-process-count) cp/ffmpeg-max-processes)
      (apply 'start-process "dired-ffmpeg"
             (generate-new-buffer-name "*dired-ffmpeg*")
             "ffmpeg"
             (split-string-and-unquote shell-args))
    (run-at-time cp/ffmpeg-process-recount-interval
                 nil
                 (cp/maybe-call-ffmpeg shell-args))))

;; TODO - 1. process buffer does not update like ffmpeg run in the
;;           terminal.

;;        2. don't make FORMAT arg compulsory - if omitted, create
;;           unique file name based on original, with same extension
;;           (`make-temp-name')

(defun cp/dired-do-ffmpeg (input-args output-args format)
  "In Dired buffers, runs `ffmpeg' on each marked file, or, if no
marks are present, on the file at point.

Each input file will result in one output file. The output file
will have the same file name but FORMAT as the extension.

This command thus uses barely a fraction of what ffmpeg can do,
but hopefully it covers any and every one-file-in-one-file out
usage, which is most common for the author.

INPUT-ARGS and OUTPUT-ARGS can be left blank (\"\"). FORMAT is
presently necessary, and must not be the same as the input file.

ffmpeg must be installed on your system, of course."
  (interactive "sInput arguments (optional) : \nsOutput arguments (optional) : \nsOutput format : ")
  (let ((files (-map #'shell-quote-argument
                     (dired-get-marked-files t))))
    (--map
     (let* ((fbasename  (file-name-base it))
            (new-name   (concat fbasename "." format))
            (args       (concat input-args " "
                                "-i " it " "
                                output-args " "
                                new-name)))
       ;; (with-output-to-temp-buffer "cp/dired-do-ffmpeg"
       (cp/maybe-call-ffmpeg args)
       ;; )
     )
     files)))

;; (with-current-buffer "log"
;;   (goto-char (point-max))
;;   (insert
;;    (concat
;;     (propertize (format-time-string "%FT%T%z") 'face 'bold)
;;     "\n"))
;;   (start-file-process-shell-command "ls" "log" "ls -g ~/Urecord*")
;;   (insert
;;    (concat
;;     "\n(cp/ffmpeg-process-count) - "
;;     (number-to-string (cp/ffmpeg-process-count))
;;     "\n(process-list) - \n"
;;     (apply 'concat "  " (-interpose "\n  " (-map 'process-name (process-list))))
;;     "\ntimer-list - \n"
;;     (apply 'concat "  " (-interpose "\n  " (-map 'prin1-to-string timer-list)))
;;     "\n\n"))
;;   (set-window-point
;;    (get-buffer-window "log")
;;    (point-max))
;;   (recenter))

;; (temp-buffer-window-show "*dired-ffmpeg*")

(defun cp/dired-do-ediff (&optional format)
  "Ediff (first two) marked files."
  (interactive)
  (let ((files     (dired-get-marked-files t))
        (files-dir default-directory))
    ;; 2018-04-08T11:31:20+0530 TODO - if there is only one marked
    ;; file, check the other window for a marked file and ediff with
    ;; that.
    (if (= 1 (length files))
        (progn
          (other-window 1)
          (let ((files2     (dired-get-marked-files t))
                (files2-dir default-directory))
            (if files2
                (ediff (expand-file-name
                        (concat files-dir (car files)))
                       (expand-file-name
                        (concat files2-dir (car files2)))))))
      ;; otherwise, expect two marked files in the buffer and ediff them
      (ediff (car files) (cadr files)))))

;; (defvar count 0)

;; (defun timer-test (count)
;;   (if (= count 5)
;;       (progn
;;         (message "count %s - finito" count)
;;         (cancel-timer test-timer))
;;     (progn
;;       (message "incrementing count - %s" count)
;;       (setq count (1+ count)))))

;; ;; works! -
;; (let ((a 1) (b 2))
;;   (setq test-timer
;;         (run-at-time 1 nil `(lambda () (message "%d" (+ ,a ,b))))))

;; ;; works
;; (let ((a 1))
;;   (setq my-timer
;;         (run-with-timer 5 5 (lambda ()
;;                               (message "I cancelled my-timer")
;;                               (cancel-timer my-timer)))))

;; ;; works
;; (setq my-timer
;;       (run-with-timer 5 5 (lambda ()
;;                             (message "I cancelled my-timer")
;;                             (cancel-timer my-timer))))

;; ;; works
;; (defvar test t)
;; (defun process-queue ()
;;   (if test
;;       (progn
;;         (message "process-queue says hi")
;;         (run-with-timer 5 nil 'process-queue))
;;     (message "process-queue says bye")))

;; (let ((a 1) (b 2) (time 5))
;;   (setq test-timer
;;         (run-at-time 5 5
;;                      `(lambda ()
;;                         (message "test-timer - %d" ,a)
;;                         (setq a (1+ ,a))))))

;; (progn
;;   (run-at-time "2 sec" nil (lambda () (message "bye!")))
;;   (message "hi!"))

(defun cp/nth-filename-this-line (&optional count)
  (let ((file (buffer-substring-no-properties
               (progn
                 (beginning-of-line)
                 (search-forward "/" nil t count)
                 (forward-char -1)
                 (point))
               (or (progn
                     (re-search-forward " und /" (point-at-eol) t)
                     (re-search-backward " " nil t 2))
                   (progn
                     (re-search-forward " sind verschieden." (point-at-eol) t)
                     (re-search-backward " " nil t 2))
                   (- (point-at-eol) 1)))))
    (if (file-exists-p file)
        file
      (replace-regexp-in-string ": " "/" file))))

(defun cp/copy-to-ehdd ()
  "Copy files using file names from diff(1) output."
  (interactive)
  (re-search-forward "^Nur in /media/alpha/")
  (let* ((file (cp/nth-filename-this-line))
         (dest (replace-regexp-in-string "^/media/alpha/"
                                         "/media/kashish-wd-1tb/alpha/"
                                         file)))
    (forward-line)
    (if (file-directory-p file)
        (copy-directory file dest t t t)
      (copy-file file dest nil t t t))))

(defun cp/diff-ediff ()
  (interactive)
  (beginning-of-line)
  (let ((file1 (cp/nth-filename-this-line))
        (file2 (cp/nth-filename-this-line 2)))))

(defun cp/diff-show-file ()
  (interactive)
  (let ((file (cp/nth-filename-this-line)))
    (dired-jump t file)
    (message file)))

(with-eval-after-load 'diff-mode
  (define-key diff-mode-map (kbd "C-o") 'cp/diff-show-file))

(defun cp/dired-update-all ()
  (mapc (lambda (b)
          (with-current-buffer b
            (if (equal major-mode 'dired-mode)
                (find-alternate-file (dired-current-directory)))))
        (buffer-list)))

(defun cp/done-matches (&optional count)
  (interactive "P")
  (let ((case-fold-search nil)
        (count (if count count 1)))
    (flet ((done-matches-op
            ()
            (when (re-search-forward "^/media/kashish-wd-1tb/alpha/kashish/new-stuff/7-music/playerdump/")
              (let* ((para-start (progn
                                   (re-search-backward "^[0-9]+ bytes each:$")
                                   (point)))
                     (matches (progn (forward-line 1)
                                     (count-lines (point)
                                                  (progn
                                                    (forward-paragraph)
                                                    (point))))))
                (when (= matches 2)
                  (delete-region (point)
                                 (progn
                                   (forward-paragraph -1)
                                   (point))))))))
      (if (= count 0)
          (while (not (eobp))
            (done-matches-op))
        (dotimes (i count)
          (done-matches-op))))))

(defun cp/change-all-units ()
  "for fdupes output"
  (interactive)
  (while (re-search-forward "^[0-9]+" nil t)
    (shell-command-on-region (point-at-bol) (point) "numfmt --to=iec-i --suffix=B" nil t)
    (forward-word)
    (delete-region (point) (progn (forward-word) (point)))))

(defun cp/launch-file-archive ()
  (interactive)
  (launch-file
   (concat
    default-directory
    (aref (archive-get-descr) 0))))

;; (define-key archive-mode-map (kbd "j") 'cp/launch-file-archive)

;; 2018-02-28T21:00:57+0530
(defun cp/corresponding-text-file ()
  (interactive)
  (save-excursion
    (end-of-line)
    (if (derived-mode-p 'dired-mode)
        ;; 2018-08-05T02:01:26+0530 - support directories too
        (let* ((file-or-dir (dired-file-name-at-point))
               (file        (if (file-directory-p file-or-dir)
                                (replace-regexp-in-string "/$" "" file-or-dir)
                              file-or-dir)))
          (find-file (concat file ".txt"))))))

;; 2018-06-23T09:57:34+0530
;; FIXME - probably very fragile, assumes a lot about dired-actual-switches
;; BUG - viewing hidden files is okay, but hiding them again never
;;       seems to work without a find-alternate-file, which in turn
;;       never seems to work non-interactively. Parts of dired-revert
;;       seem relevant to this.
(defun cp/dired-toggle-hidden ()
  "Toggle visibility of hidden files."
  (interactive)
  (if (string-match-p "-A" dired-listing-switches)
      (setq dired-actual-switches
            (replace-regexp-in-string "\\(-A\\).*\\'"
                                      "-"
                                      dired-listing-switches
                                      nil nil 1))
    (setq dired-actual-switches
          (replace-regexp-in-string "\\(-\\).*\\'"
                                    "-A"
                                    dired-listing-switches
                                    nil nil 1)))
  ;; (revert-buffer)
  (dired-revert)
  ;; (find-alternate-file (dired-current-directory)) ;; no good at all

  ;; no good either
  ;; (let ((directory (dired-current-directory)))
  ;;   (kill-this-buffer)
  ;;   (dired directory))
  )

;; 2018-07-09T23:22:17+0530
;; a little buggy wrt clicks
(defun cp/dired-launch-or-open (event)
  (interactive "e")
  ;; if point is on a folder, open it with dired
  ;; otherwise, call launch-files-dired
  (if (directory-name-p (dired-file-name-at-point))
      (dired-find-file)
    (launch-files-dired nil (dired-get-marked-files))))
