(require 'cp)

;;;; Navigation improvements

(add-to-list 'load-path "~/.emacs.d/elisp-git/window-numbering/")

;; Vim-style prompt-less tag jumping
;; http://stackoverflow.com/questions/12074897/automatically-jump-to-tag-in-emacs
(global-unset-key (kbd "M-t"))
(defun find-tag-no-prompt-other-window ()
  "Jump to the tag at point in the other window without prompting."
  (interactive)
  (find-tag-other-window (find-tag-default)))
(defun find-tag-no-prompt ()
  "Jump to the tag at point without prompting."
  (interactive)
  (find-tag (find-tag-default)))

(cp-set-keys
 :bindings
 `((,(kbd "M-t M-t") find-tag-no-prompt-other-window)
   (,(kbd "M-t M-g") find-tag-no-prompt)
   (,(kbd "M-t M-p") pop-tag-mark)
   (,(kbd "M-t M-f") find-tag)
   (,(kbd "M-t M-s") tags-search)
   (,(kbd "M-t M-c") tags-loop-continue)))

;; (require 'view)
;; (cp-set-keys
;;  :bindings
;;  `((,(kbd "C-v") View-scroll-half-page-forward)
;;    (,(kbd "M-v") View-scroll-half-page-backward)))
;; ;; (cp-set-keys
;; ;;  :bindings
;; ;;  `((,(kbd "C-v") scroll-up)
;; ;;    (,(kbd "M-v") scroll-down)))

;;;; Need to make maps for
;;;;    mark-* commands (-sexp, -page, -word, etc)
;;;;    mark ring

;;;; Other custom keys
(cp-set-keys
 :bindings
 `((,(kbd "C-x C-1") delete-other-windows)
   (,(kbd "C-x C-2") split-window-below)
   (,(kbd "C-x C-3") split-window-right)
   (,(kbd "C-x C-0") delete-window)
   (,(kbd "C-x C-4 C-f") find-file-other-window)
   ;; (,(kbd "C-`") point-to-register)
   ;; (,(kbd "C-'") jump-to-register)
   (,(kbd "C-x C-d") dired-jump)
   ;; I usually keep Super for the window manager and global hotkeys...
   (,(kbd "s-i") imenu)
   ;; keypad

   ;; set this to run whatever command is bound to C-c C-c, or maybe just C-c
   ;; see (info "(elisp)Translation Keymaps")
   (,(kbd "<kp-divide>"))
   (,(kbd "<kp-multiply>") cp/copy-buffer)
   (,(kbd "<kp-subtract>") erc-track-switch-buffer)
   (,(kbd "S-<kp-subtract>") cp-kill-buffer)

   (,(kbd "<kp-home>")     delete-other-windows)
   (,(kbd "<kp-up>")       split-window-below)
   (,(kbd "<kp-prior>")    split-window-right)

   (,(kbd "<kp-begin>")    ido-mini)

   (,(kbd "<kp-add>")      save-buffer)
   (,(kbd "S-<kp-add>")    (lambda () (interactive) (call-interactively (ert-run-tests-interactively t))))
   (,(kbd "<kp-enter>")    eval-defun)
   (,(kbd "S-<kp-enter>")  eval-buffer)
   (,(kbd "C-S-<kp-enter>") toggle-debug-on-error)

   (,(kbd "<kp-end>")      select-window-1)
   (,(kbd "<kp-down>")     select-window-2)
   (,(kbd "<kp-next>")     select-window-3)

   (,(kbd "<kp-insert>")   timeclock-list)
   (,(kbd "C-<kp-insert>") timeclock-report)
   (,(kbd "<kp-delete>")   abort-recursive-edit)))

(with-eval-after-load 'dired
  (define-key dired-mode-map (kbd "b") 'dired-up-directory)
  ;; (add-hook 'dired-mode-hook 'visual-line-mode)
  ;(global-set-key (kbd "C-,") 'string-rectangle 'TAB)
)
(global-unset-key (kbd "C-x d"))

;; (require 'dired-toggle-sudo)
;; (define-key dired-mode-map (kbd "C-c C-s") 'dired-toggle-sudo)
;; (eval-after-load 'tramp
;;   '(progn
;;      ;; Allow to use: /sudo:user@host:/path/to/file
;;      (add-to-list 'tramp-default-proxies-alist
;;                   '(".*" "\\`.+\\'" "/ssh:%h:"))))
(global-set-key (kbd "C-x df") 'delete-frame)
;;;; make-frame key is after Helm's config

;(setq compilation-read-command nil)
(defun cp-open-init ()
  "Open the init.el file."
  (interactive)
  (find-file "~/.emacs.d/init.el"))
;; todo - store current file name → kill the buffer →
;; find-file-literally with the stored filename. also, find out if you
;; can open it without modes but with the proper encoding.
(defun cp-fcf-literally ()
  "Find the current file literally. Like find-file-literally but
  does not prompt for a file name."
  (interactive)
  (find-file-literally (buffer-file-name)))
(defun cp-kill-buffer ()
  "Kill the current buffer without prompting."
  (interactive)
  (kill-buffer nil))

(autoload 'ibuffer "ibuffer" "ibuffer" t)
(global-set-key (kbd "C-x C-b") 'ibuffer)
(with-eval-after-load 'ibuffer
  (cp-set-keys
   :unset t
   :keymap ibuffer-mode-map
   :bindings
   `((,(kbd "m"))
     (,(kbd "U"))))
  (cp-set-keys
   :keymap ibuffer-mode-map
   :bindings
   `((,(kbd "m f") ibuffer-mark-by-file-name-regexp)
     (,(kbd "m m") ibuffer-mark-by-mode-regexp)
     (,(kbd "m n") ibuffer-mark-by-name-regexp)
     (,(kbd "M")   ibuffer-mark-forward)
     (,(kbd "r")   ibuffer-do-replace-regexp)
     (,(kbd "U")   ibuffer-unmark-all)))
  (add-hook 'ibuffer-mode-hook 'ibuffer-set-filter-groups-by-mode))

;; (require 'win-switch)
;; (global-set-key "\C-xo" 'win-switch-dispatch)
;; (win-switch-setup-keys-ijkl)
;; (setq win-switch-idle-time 0.5)
;; (setq win-switch-other-window-first nil)
;; ;(setq win-switch-other-window-first nil)

(require 'window-numbering)
(window-numbering-mode 1)

(defun cp-maximize-window ()
  "Run maximize-window if more than one window is present."
  (interactive)
  (if (> (length (window-list)) 1)
      (maximize-window)))

(defun cp-minimize-window ()
  "Run minimize-window if more than one window is present."
  (interactive)
  (if (> (length (window-list)) 1)
      (minimize-window)))

(with-eval-after-load 'doc-view
  (define-key doc-view-mode-map (kbd "=") 'doc-view-fit-height-to-window))

(with-eval-after-load 'view-mode
  (define-key view-mode-map (kbd "x") nil)
  (define-key view-mode-map (kbd "<backspace>") 'scroll-down-command))
